#include "Frontend.h"
#include "convert.h"

enum
{
  SOCKET_ID
};

frontend::frontend() : wxFrontend(NULL) {

	// initializing the socket and its event handling
	SockClt = new wxSocketClient();
	SockClt->SetEventHandler(*this, SOCKET_ID);
	SockClt->SetNotify(wxSOCKET_CONNECTION_FLAG | wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG);
	SockClt->Notify(true);
	Connect(wxEVT_SOCKET, wxSocketEventHandler( frontend::OnSocketEvent ), NULL, this);	
	
	// Host configuration - should be held in external config file
	wxString hostname = wxGetTextFromUser(
    _("Adresse des monitord-Servers:"),
    _("Connect ..."),
    _("127.0.0.1"));

	wxString service = wxGetTextFromUser(
    _("Port des monitord-Servers:"),
    _("Connect ..."),
    _("9333"));
	
	Host.Hostname(hostname);
	Host.Service(service);
	
	SockClt->Connect(Host, false);

	ClientCheckBox->SetValue(true);
	ServerCheckBox->SetValue(true);
}

frontend::~frontend() {
	SockClt->Destroy();
}

/**
* A socket event has been issued - switch and debugging only as other methods are called from here
*/
void frontend::OnSocketEvent(wxSocketEvent& event)
{
	switch(event.GetSocketEvent()) {
		case wxSOCKET_INPUT      : frontend::OnSocketInput(event); break;
		case wxSOCKET_LOST       : frontend::OnSocketLost(event); break;
		case wxSOCKET_CONNECTION : frontend::OnSocketConnection(event); break;
		default                  : break;
	}
}

/**
* Connection lost or could not be established
*/
void frontend::OnSocketLost(wxSocketEvent& event) {
	wxMessageBox("Verbindung konnte nicht hergestellt werden oder wurde beendet!", "Socket-Event", wxICON_INFORMATION);
	OtherLog->AppendText(wxString("\nwxFrontend: Verbindung wurde nicht hergestellt oder abgebrochen.\n"));
}

/**
* Connection established
*/
void frontend::OnSocketConnection(wxSocketEvent& event) {

}

/**
* Input from socket
*/
void frontend::OnSocketInput(wxSocketEvent& event) {
	// needed input streams
	SocketInStream = new wxSocketInputStream(*SockClt);
	TextInStream = new wxTextInputStream(*SocketInStream);
	
	// reading one line and tokenizing it into an array auf strings
	wxString line = TextInStream->ReadLine();
	wxStringTokenizer StringParts(line, _(":"));
	wxArrayString *DataFromSocket = new wxArrayString();
	while ( StringParts.HasMoreTokens() )
	{
		DataFromSocket->Add(StringParts.GetNextToken());
	}
		
	// looking for type of information gotten from socket
	if (DataFromSocket->Item(0) == "300") {
		OnZVEI(DataFromSocket);
	} else if (DataFromSocket->Item(0) == "310") {
		OnFMS(DataFromSocket);
	} else if (DataFromSocket->Item(0) == "320") {
		OnPOCSAG(DataFromSocket);
	} else if (DataFromSocket->Item(0) == "101") {
		OnFailure(DataFromSocket);
	} else {
		if(ServerCheckBox->GetValue()) {
			OtherLog->AppendText(line + "\n");
		}
	}
}

void frontend::OnZVEI( wxArrayString *DataFromSocket ) {
		std::string outputstring = _("Kein Text");
		if(DataFromSocket->GetCount() > 5) {
			std::string inputstring = DataFromSocket->Item(5).c_str();
			bool err = convertHexToString(inputstring, outputstring);
		} else {
			outputstring = "Kein Text";
		}
		ZVEILog->AppendText("Zeit: " + DataFromSocket->Item(1) + ", Kanal: " + DataFromSocket->Item(2) + ", Schleife: " + DataFromSocket->Item(3) + ", Typ: " + DataFromSocket->Item(4) + "\nText: '" + outputstring + "'\n\n");
}

void frontend::OnFMS( wxArrayString *DataFromSocket ) {
		std::string outputstring = "Kein Text";
		if(DataFromSocket->GetCount() > 8) {
			std::string inputstring = DataFromSocket->Item(8).c_str();
			bool err = convertHexToString(inputstring, outputstring);
		} else {
			outputstring = "Kein Text";
		}
		FMSLog->AppendText("Zeit: " + DataFromSocket->Item(1) + ", Kanal: " + DataFromSocket->Item(2) + ", Kennung: " + DataFromSocket->Item(3) + ", Status: " + DataFromSocket->Item(4) + ", Baustufe: " + DataFromSocket->Item(5) + ", Richtung: " + DataFromSocket->Item(6) + ", TKI: " + DataFromSocket->Item(7) + "\nText: '" + outputstring + "'\n\n");
}

void frontend::OnPOCSAG( wxArrayString *DataFromSocket ) {
		std::string outputstring = "Kein Text";
		if(DataFromSocket->GetCount() > 5) {
			std::string inputstring = DataFromSocket->Item(5).c_str();
			bool err = convertHexToString(inputstring, outputstring);
		} else {
			outputstring = "Kein Text";
		}
		POCSAGLog->AppendText("Zeit: " + DataFromSocket->Item(1) + ", Kanal: " + DataFromSocket->Item(2) + ", RIC: " + DataFromSocket->Item(3) + ", Subadresse: " + DataFromSocket->Item(4)	+ "\nText: '" + outputstring + "'\n\n");
}

void frontend::OnFailure( wxArrayString *DataFromSocket ) {
	std::string outtext = "FAIL:\n";
	for(int count=0;count < DataFromSocket->GetCount();count++) {
		outtext = outtext + DataFromSocket->Item(count) + ", ";
	}
	OtherLog->AppendText(outtext + "\n\n");
}

void frontend::OnInquiryBtn( wxCommandEvent& event ) {
	SocketOutStream = new wxSocketOutputStream(*SockClt);
	TextOutStream = new wxTextOutputStream(*SocketOutStream);
	TextOutStream->WriteString(_("210\n"));
	if(ClientCheckBox->GetValue()) {
		OtherLog->AppendText(_("210\n"));
	}
}

void frontend::OnLogoffBtn( wxCommandEvent& event ) {
	SocketOutStream = new wxSocketOutputStream(*SockClt);
	TextOutStream = new wxTextOutputStream(*SocketOutStream);
	TextOutStream->WriteString(_("299\n"));
	if(ClientCheckBox->GetValue()) {
		OtherLog->AppendText(_("299\n"));
	}
}

void frontend::OnKeepaliveBtn( wxCommandEvent& event ) {
	SocketOutStream = new wxSocketOutputStream(*SockClt);
	TextOutStream = new wxTextOutputStream(*SocketOutStream);
	TextOutStream->WriteString(_("202\n"));
	if(ClientCheckBox->GetValue()) {
		OtherLog->AppendText(_("202\n"));
	}
}

void frontend::OnChannelInfoBtn( wxCommandEvent& event ) {
	SocketOutStream = new wxSocketOutputStream(*SockClt);
	TextOutStream = new wxTextOutputStream(*SocketOutStream);
	TextOutStream->WriteString(_("203\n"));
	if(ClientCheckBox->GetValue()) {
		OtherLog->AppendText(_("203\n"));
	}
}

void frontend::OnLoginBtn( wxCommandEvent& event ) {
	SocketOutStream = new wxSocketOutputStream(*SockClt);
	TextOutStream = new wxTextOutputStream(*SocketOutStream);
	
	std::string inputstring = Username->GetValue().c_str();
	std::string outputstring;
	bool err = convertStringToHex(inputstring, outputstring);
	wxString SocketOutputString = "220:" + outputstring;
	inputstring = Passwd->GetValue().c_str();
	err = convertStringToHex(inputstring, outputstring);
	SocketOutputString = SocketOutputString + ":" + outputstring + "\n";
	
	TextOutStream->WriteString(SocketOutputString);
	
	if(ClientCheckBox->GetValue()) {
		OtherLog->AppendText(SocketOutputString);
	}
}
